package ru.ibs.tests;

import io.cucumber.java.After;
import io.cucumber.java.ru.Дано;
import io.cucumber.java.ru.И;
import io.cucumber.java.ru.Тогда;

import org.openqa.selenium.By;
import org.openqa.selenium.Keys;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;

import org.openqa.selenium.support.ui.ExpectedConditions;
import org.openqa.selenium.support.ui.WebDriverWait;
import ru.ibs.framework.managers.DriverManager;
import ru.ibs.framework.pages.CatalogPage;
import ru.ibs.framework.pages.ListingPage;
import ru.ibs.framework.pages.MainPage;

import java.time.Duration;
import java.util.List;

import static org.junit.Assert.*;

public class SearchSteps {

    WebDriver driver = DriverManager.getDriverManager().getDriver();
    MainPage mainPage = new MainPage(driver);
    CatalogPage catalogPage = new CatalogPage(driver);
    ListingPage listingPage = new ListingPage(driver);
    String savedProductTitle;

    @Дано("Открываем сайт regard")
    public void openSite() {
        driver.get("https://regard.ru/");
        driver.manage().window().maximize();
        catalogPage.closeCookieBannerIfPresent();
    }

    @И("Открываем меню Каталог")
    public void openCatalog() {
        mainPage.openCatalog();
    }

    @И("Выбираем раздел {string}")
    public void chooseCategory(String category) {
        catalogPage.selectCategory(category);
    }

    @И("Выбираем подраздел {string}")
    public void chooseSubCategory(String sub) {
        catalogPage.selectSubCategory(sub);
    }

    @И("Задаем цену от {string}")
    public void setPrice(String price) {
        listingPage.setMinPrice(price);
    }

    @И("Фильтруем по производителю {string}")
    public void filterByManufacturer(String name) {
        if (name.equalsIgnoreCase("Gigabyte")) {
            listingPage.selectGigabyte();
        }
    }

    @И("Проверяем, что товаров не более {int}")
    public void checkNumberOfProducts(int max) {
        assertTrue(listingPage.getAllProducts().size() <= max);
    }

    @И("Сохраняем наименование первого товара")
    public void saveFirstProductName() {
        savedProductTitle = listingPage.getFirstProductTitle();
    }

    @И("Ищем товар по сохраненному имени")
    public void searchSavedProduct() {
        WebElement searchInput = driver.findElement(By.id("searchInput"));
        searchInput.clear();
        searchInput.sendKeys(savedProductTitle);
        searchInput.sendKeys(Keys.ENTER);

        // Ждем, пока появится хотя бы один товар на странице результатов поиска
        new WebDriverWait(driver, Duration.ofSeconds(10)).until(
                ExpectedConditions.visibilityOfElementLocated(By.cssSelector("div.Card_wrap__hES44"))
        );
    }

    @Тогда("На странице только один товар")
    public void waitForProductsCountChange() {
        WebDriverWait wait = new WebDriverWait(driver, Duration.ofSeconds(10));
        int oldCount = listingPage.getAllProducts().size();
        wait.until(driver -> driver.findElements(By.cssSelector("div.Card_wrap__hES44")).size() != oldCount);
    }

    @И("Название товара соответствует сохраненному")
    public void titleShouldMatchSaved() {
        assertEquals(savedProductTitle, listingPage.getFirstProductTitle());
    }

    @After
    public void tearDown() {
        DriverManager.getDriverManager().quitDriver();
    }
}
